@@ -20,11 +20,13 @@
 
 import threading
 from .devices import Device, Value, NullValue, TecpelDMM8061, XLS200, KernPCB, BS600
+import si
 
 class DeviceManager(object):
-    """DeviceManager object manages devices.
+    """The DeviceManager manages devices.
 
-    It provides methods to add devices and get their most recent values.
+    It provides methods to add or delete devices,
+    get their most recent measurement values and various other things.
 
     """
     
@@ -41,10 +43,8 @@ class DeviceManager(object):
         pass
 
     def _checkConfig(self, config):
-        assert config.deviceName in self._validDevices
-        assert isinstance(config.relationship, tuple) and (len(config.relationship) == 2 or len(config.relationship) == 3)
-        assert isinstance(config.relationship[0], str) or isinstance(config.relationship[0], float)
-        assert isinstance(config.relationship[1], dict)
+        assert config["deviceName"] in self._validDevices
+        assert isinstance(config["parent"], str) or isinstance(config["parent"], float)  # FIXME: Check for deviceID object when implemented
 
     def _updateRawValues(self):
         if self._running:
@@ -56,10 +56,10 @@ class DeviceManager(object):
     def _getLinearFunction(self, value1, value2):
         """Return a linear function, which contains value1 and value2.
 
-        :param value1: tuple of x,y coordinates
-        :type value1: tuple x,y
-        :param value2: tuple of x,y coordinates
-        :type value2: tuple x,y
+        :param value1: Tuple of x,y coordinates
+        :type value1: Tuple x,y
+        :param value2: Tuple of x,y coordinates
+        :type value2: Tuple x,y
 
         """
         try:
@@ -77,17 +77,34 @@ class DeviceManager(object):
         return result
 
     def isValidDevice(self, deviceName):
-        """Tests whether deviceName is a valid device."""
+        """Tests whether deviceName is a valid device.
+
+        :param deviceName: DeviceName (e.g. XLS200, TecpelDMM8061 etc.)
+        :type deviceName: String
+        :returns: Whether deviceName is a valid deviceName
+        :rtype: Boolean
+
+        """
 
         return deviceName in self._validDevices
 
     def getValidDevices(self):
-        """Returns a list of valid devicenames"""
+        """Returns a list of valid deviceNames.
+        
+        :returns: List of valid deviceNames
+        :rtype: List of strings
+
+        """
 
         return self._validDevices
 
     def getAllDeviceIDs(self):
-        """Returns a complete list of registered device ID's"""
+        """Returns a complete list of registered deviceIDs.
+
+        :returns: Complete list of registered deviceIDs
+        :rtype: List of deviceIDs
+
+        """
 
         ids = []
         for id in self.configs:
@@ -96,7 +113,12 @@ class DeviceManager(object):
         return ids
 
     def getAvailiablePorts(self):
-        """Returns a list of valid and availiable serial ports."""
+        """Returns a list of valid and availiable serial ports.
+
+        :returns: List of valid and availiable serial ports
+        :rtype: List of strings
+
+        """
 
         import serial.tools.list_ports
         portsTuple = serial.tools.list_ports.comports()
@@ -108,12 +130,67 @@ class DeviceManager(object):
         return tuple(portsList)
 
     def getStatus(self):
-        """Returns the status of the devicemanager."""
+        """Returns the status of the deviceManager.
+        
+        True if deviceManager is running, otherwise False.
+
+        :returns: Status of deviceManager
+        :rtype: Boolean
+
+        """
 
         return self._running
 
-    def openWithConfig(self, config):
-        """Open a device with a config object. Returns a deviceID."""
+    def openDevice(self, deviceName, serialPort, *args, **kwargs):
+        """Open a device.
+        
+        :param deviceName: Classname of the device to openDevice
+        :type deviceName: String
+        :param serialPort: Serial port on which the device sits
+        :type serialPort: String
+        :param args: args to pass to the devices init method
+        :param kwargs: kwargs to pass to the devices init method
+        :returns: DeviceID of opened device
+        :rtype: :class:`DeviceID`
+
+        """
+
+        return self._openWithConfig({"deviceName":deviceName,
+                                    "parent":serialPort,
+                                    "args":args,
+                                    "kwargs":kwargs})
+
+    def openSubdevice(self, deviceName, parentDeviceID, inputNumber, *args, **kwargs):
+        """Open a device as subdevice of the device with parentDeviceID.
+        
+        :param deviceName: Classname of the device to openDevice
+        :type deviceName: String
+        :param parentDeviceID: DeviceID of the parent device (the multiboxdevice)
+        :type parentDeviceID: DeviceID
+        :param inputNumber: Input the device sits on
+        :type inputNumber: Int
+        :param args: args to pass to the devices init method
+        :param kwargs: kwargs to pass to the devices init method
+        :returns: DeviceID of opened device
+        :rtype: :class:`DeviceID`
+
+        """
+
+        return self._openWithConfig({"deviceName":deviceName,
+                                    "parent":parentDeviceID,
+                                    "inputNumber":inputNumber,
+                                    "args":args,
+                                    "kwargs":kwargs})
+
+    def _openWithConfig(self, config):
+        """Open a device with a config dictionary. Returns a deviceID.
+
+        :param config: Config dictionary
+        :type config: Dictionary
+        :returns: DeviceID of opened device
+        :rtype: :class:`DeviceID`
+
+        """
 
         import time
 
@@ -122,18 +199,20 @@ class DeviceManager(object):
         self._checkConfig(config)
         id = time.time()
 
-        parentID = config.relationship[0]
+        parentID = config["parent"]
+
+        config["subDevices"] = {}
 
         if isinstance(parentID, str):
-            device = eval(config.deviceName).openRS232(config.relationship[0], *config.args, **config.kwargs)
+            device = eval(config["deviceName"]).openRS232(config["parent"], *config["args"], **config["kwargs"])
 
-        elif isinstance(parentID, float):
+        elif isinstance(parentID, float): # FIXME: Check for deviceID object when implemented
             assert self.devices[parentID] # FIXME: use another error (e.g. WrongIDError)
 
-            self.devices[parentID].openDevice(eval(config.deviceName),
-                input = config.relationship[2], *config.args, **config.kwargs)
-            device = self.devices[parentID].getDevice(input = config.relationship[2])
-            self.configs[parentID].relationship[1][id] = config.relationship[2]
+            self.devices[parentID].openDevice(eval(config["deviceName"]),
+                input = config["inputNumber"], *config["args"], **config["kwargs"])
+            device = self.devices[parentID].getDevice(input = config["inputNumber"])
+            self.configs[parentID]["subDevices"][id] = config["inputNumber"]
 
         self.devices[id] = device
         self.configs[id] = config
@@ -141,38 +220,54 @@ class DeviceManager(object):
         return id
 
     def closeDevice(self, deviceID):
+        """Close device with deviceID.
+        
+        :param deviceID: DeviceID of device to close
+        :type deviceID: DeviceID
+
+        """
+
         assert self._running == False # FIXME: use another error
 
         config = self.configs[deviceID]
 
-        if isinstance(config.relationship[0], str):
+        if isinstance(config["parent"], str):
             import copy
-            relCopy = copy.copy(config.relationship[1])
+            relCopy = copy.copy(config["subDevices"])
             # prevent RuntimeError: dictionary changed size during iteration
             
             for subdeviceID, inputNumber in relCopy.iteritems():
                 self.closeDevice(subdeviceID)
 
+            self.devices[deviceID].close()
+
             del(self.devices[deviceID])
             del(self.configs[deviceID])
 
-        elif isinstance(config.relationship[0], float):
-            parentID = config.relationship[0]
-            self.getDevice(parentID).closeDevice(input=config.relationship[2])
+        elif isinstance(config["parent"], float): # FIXME: Check for deviceID object when implemented
+            parentID = config["parent"]
+            self.getDevice(parentID).closeDevice(input=config["inputNumber"])
             del(self.devices[deviceID])
             del(self.configs[deviceID])
-            del(self.configs[parentID].relationship[1][deviceID])
+            del(self.configs[parentID]["subDevices"][deviceID])
 
     def closeEmptyMultiboxDevices(self):
+        """Close all MultiboxDevices without subDevices."""
+
         for i in self.getAllDeviceIDs():
             config = self.configs[i]
-            if len(config.relationship[1]) == 0 and config.deviceName == "XLS200":
+            if len(config["subDevices"]) == 0 and config["deviceName"] == "XLS200":
                 self.closeDevice(i)
 
     def getLastRawValue(self, deviceID):
         """Returns the last raw value of device with ID deviceID.
 
         If there is no last value (last value == None), it returns a NullValue.
+
+        :param deviceID: deviceID (e.g. XLS200, TecpelDMM8061 etc.)
+        :type deviceID: DeviceID
+        :returns: Last Value object of device with deviceID
+        :rtype: Value
         
         """
 
@@ -190,14 +285,17 @@ class DeviceManager(object):
     def getCalibratedLastRawValue(self, deviceID, calibration, unit=None):
         """Return a calibrated value from the getLastRawValue method.
         
-        :param deviceID: device id
-        :type deviceID: float
-        :param calibration: calibration tuple
-        :type calibration: tuple
+        :param deviceID: DeviceID
+        :type deviceID: DeviceID
+        :param calibration: Calibration tuple
+        :type calibration: Tuple
+        :param unit: Unit or None
+        :type unit: String or None
 
-        The calibration tuple must contain either two integers (m, c)
-        or two tuples containing two points on the linear function,
-        used to compute m and c. (of a function f(x) = mx+c)
+        The calibration tuple must contain either two integers m, c
+        to create a function of style f(x) = m x + c
+        or two tuples containing two points on such a linear function,
+        used to compute m and c.
 
         """
         try:
@@ -207,29 +305,30 @@ class DeviceManager(object):
             linearFunction = resultFunction
 
         class __Value(Value):
-            displayedValue = None
+            value = None
             unit = None
-            factor = (pow(10, 0),"")
 
             def getDisplayedValue(self):
-                return self.displayedValue
+                x = si.getNumberPrefix(self.value)
+                return x[0]
 
             def getUnit(self):
                 return self.unit
 
             def getFactor(self, type="value"):
+                x = si.getNumberPrefix(self.value)
+                
                 if type == "value":
-                    return self.factor[0]
+                    return si.getFactor(x[1])
                 elif type == "prefix":
-                    return self.factor[1]
+                    return x[1]
 
         rv = self.getLastRawValue(deviceID)
         if rv == None: return None
 
         result = __Value()
 
-        result.displayedValue = linearFunction(rv.getDisplayedValue())
-        result.factor = (rv.getFactor(type="value"), rv.getFactor(type="prefix"))
+        result.value = linearFunction(rv.getDisplayedValue() * rv.getFactor())
         result._time = rv.getTime()
 
         if unit: result.unit = unit
@@ -238,6 +337,15 @@ class DeviceManager(object):
         return result
 
     def getDevice(self, deviceID):
+        """Return device with deviceID.
+        
+        :param deviceID: DeviceID
+        :type deviceID: DeviceID
+        :returns: :class:`devices.devices.Device` with deviceID
+        :rtype: :class:`devices.devices.Device`
+
+        """
+
         try:
             return self.devices[deviceID]
 
@@ -245,7 +353,7 @@ class DeviceManager(object):
             return None
 
     def start(self):
-        """Start the devicemanager"""
+        """Start the DeviceManager."""
 
         self._stopEvent = threading.Event()
         self._thread = _GetValuesThread(self._stopEvent, self.devices, self.configs)
@@ -253,7 +361,7 @@ class DeviceManager(object):
         self._running = True
 
     def stop(self):
-        """Stop the devicemanager"""
+        """Stop the DeviceManager."""
 
         if self._running == True:
             self._stopEvent.set()
@@ -262,26 +370,6 @@ class DeviceManager(object):
             self._stopEvent = None
 
 
-class DeviceConfig(object):
-    """DeviceConfig objects can be passed to DeviceManager openWithConfig method."""
-
-    relationship = () # ("parent" deviceID or rs232 port, {"subdeviceID":inputNumber}, inputNumber)
-    deviceName = None
-    args = ()
-    kwargs = {}
-
-    def __init__(self, relationship, deviceName, *args, **kwargs):
-        assert deviceName in DeviceManager._validDevices
-        assert isinstance(relationship, tuple) and (len(relationship) == 2 or len(relationship) == 3)
-        assert isinstance(relationship[0], str) or isinstance(relationship[0], float)
-        assert isinstance(relationship[1], dict)
-
-        self.relationship = relationship
-        self.deviceName = deviceName
-        self.args = args
-        self.kwargs = kwargs
-
-
 class _GetValuesThread(threading.Thread):
     devices = {}
     configs = {}
@@ -306,8 +394,8 @@ class _GetValuesThread(threading.Thread):
         # python3 incompatibility: iteritems
         try:
             for key, config in self.configs.iteritems():
-                if isinstance(config.relationship[0], str):
-                    if len(config.relationship[1]) == 0 and config.deviceName != "XLS200":
+                if isinstance(config["parent"], str):
+                    if len(config["subDevices"]) == 0 and config["deviceName"] != "XLS200":
                             # If there are no subdevices and it's not an "empty" Multiboxdevice
                         rv = self.devices[key].getRawValue()
                         if rv:
@@ -316,7 +404,7 @@ class _GetValuesThread(threading.Thread):
                             pass
 
                     else:
-                        for subID, subInput in config.relationship[1].iteritems():
+                        for subID, subInput in config["subDevices"].iteritems():
                             rv = self.devices[key].getRawValue(input = subInput)
                             if rv:
                                 self.rawValues[subID] = rv
@@ -329,27 +417,4 @@ class _GetValuesThread(threading.Thread):
             # function is still trying to get an rawValue. (-> the devicemanager
             # is therefore not closed yet) It does no harm though,
             # as the device is going to be closed afterwards.
-            pass
-
-if __name__ == '__main__':
-    dm = DeviceManager()
-
-    a = DeviceConfig(("/dev/ttyUSB0", {}), "XLS200")
-    ida = dm.openWithConfig(a)
-    b = DeviceConfig((ida, {}, 2), "TecpelDMM8061")
-    idb = dm.openWithConfig(b)
-    c = DeviceConfig((ida, {}, 3), "BS600", typeOfValue="stable")
-    idc = dm.openWithConfig(c)
-    
-    dm.start()
-
-    import time
-
-    time.sleep(3)
-
-    while True:
-        time.sleep(0.5)
-        rv = dm.getCalibratedLastRawValue(idb, ((pow(10,-6), 21), (4 * pow(10,-6), 30)), unit="°C")
-        rvc  = dm.getLastRawValue(idc)
-        print(str(rv.getDisplayedValue() * rv.getFactor()) + " " + rv.getUnit())
-        print(str(rvc.getDisplayedValue() * rvc.getFactor()) + " " + rvc.getUnit())
\ No newline at end of file
+            pass
\ No newline at end of file
