@@ -18,8 +18,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
+import si
+
 class Device(object):
-    """This class is the base device class.
+    """The base device class.
 
     Subclasses must define at least the methods defined in this base class
     or, when applicable inherit them unmodified.
@@ -36,8 +39,8 @@ class Device(object):
     # (e.g. one of three devices, which are behind a multibox)
     _timeout = 0.1 # timeout value for reading from serial device
 
-    def __init__(self, ser, *args, **kwargs):
-        """Setup a device as new or subdevice.
+    def __init__(self, ser):
+        """Setup a device as new device or as subdevice.
 
         Assign ser to self._ser and if ser is not already openend (the device is not a sub device)
         set the in the class defined attributes and open the serial connection.
@@ -46,8 +49,8 @@ class Device(object):
         To setup the device as a subdevice simply pass the serial connection to the class constructor.
         To setup the device as a new device one has to use the openRS232 classmethod.
 
-        Subclasses may use more positional or keyword arguments.
-        It is safe to pass arbitrary *args and **kwargs.
+        :param ser: Serial connection (pyserial module)
+        :type ser: Serial
 
         """
 
@@ -66,23 +69,24 @@ class Device(object):
             self._ser.setRTS(level=self._rts)
             self._ser.setDTR(level=self._dtr)
 
-    def __del__(self):
-        """Closes the serial connection if the device ownes the serial connection (if its not a subdevice)"""
+    def close(self):
+        """Closes the serial connection if the device ownes the serial connection (if its not a subdevice)."""
 
         if self._ownSer is True:
             self._ser.close()
 
     @classmethod
     def openRS232(cls, port, *args, **kwargs):
-        """Create a new serial connection and create a new object with it.
+        """Open a new serial connection and create a Device object with it.
 
-        One has to pass the port argument, which must be a valid argument following to
-        the pyserial documentation ( http://pyserial.sourceforge.net/ ):
+        :param port: Serial port (see pyserial documentation)
+        :type port: String
+        :returns: Device object
+        :rtype: Device
 
-        - /dev/ttyUSB0 (Linux)
-        - COM1 (Windows)
-
-        For more examples see the pyserial documentation.
+        Valid port parameters are for example:
+            - /dev/ttyUSB0 (Linux)
+            - COM1 (Windows)
 
         """
 
@@ -98,20 +102,26 @@ class Device(object):
             serial=self._ser, ownser = self._ownSer)
 
     def __str__(self):
-        return "{classname} (port={serialport!s})".format(classname=self.__class__.__name__,
-            serialport=self._ser.port)
+        return "{classname} (port={serialport!s}, baudrate={serialbaudr!s})".format(
+            classname=self.__class__.__name__,
+            serialport=self._ser.port,
+            serialbaudr=self._baudrate)
 
     def isAvailable(self):
-        """Returns the device status (available or not)"""
+        """Returns the device status (available or not)
+        
+        :returns: Device status
+        :rtype: Boolean
+
+        """
 
         return self._ser.isOpen()
 
-    def getRawValue(self, timeout = 1.5):
-        """Return a Value object.
+    def getRawValue(self):
+        """Return a Value object or None.
 
-        The timeout option specifies a timeout in seconds.
-        If the timeout is reached and no value gathered,
-        None is returned
+        :returns: :class:`devices.devices.Value` or :class:`None`
+        :rtype: :class:`devices.devices.Value` or :class:`None`
 
         """
 
@@ -120,29 +130,46 @@ class Device(object):
 
 class MultiboxDevice(Device):
     """This is a special MultiboxDevice, which allows to use multiple
-    other devices on just one serial connection.
+    subdevices on just one serial connection.
 
     Every subclass must implement at least the in this class defined methods
     and the methods of the base device class.
 
     """
 
-    def openDevice(self, deviceClass, input = 1, *args, **kwargs):
-        """Open a device of deviceClass on input X
+    def openDevice(self, deviceClass, input, *args, **kwargs):
+        """Open a device of deviceClass on input.
 
-        *args and **kwargs are passed on to the devices __init__ methods.
+        \*args and \*\*kwargs are passed on to the devices __init__ methods.
+
+        :param deviceClass: Class of the subdevice
+        :type deviceClass: :class:`devices.devices.Device`
+        :param input: Input number of subdevice
+        :type input: Input
 
         """
 
         raise NotImplementedError
 
-    def closeDevice(self, input = 1):
-        """Close the device on input X"""
+    def closeDevice(self, input):
+        """Close the device on input.
+        
+        :param input: Input number of subdevice
+        :type input: Input
+
+        """
 
         raise NotImplementedError
 
-    def getDevice(self, input = 1):
-        """This returns the device object on input X"""
+    def getDevice(self, input):
+        """This returns the device object on input.
+
+        :param input: Input number of subdevice
+        :type input: Input
+        :returns: :class:`devices.devices.Device` on input
+        :rtype: :class:`devices.devices.Device`
+
+        """
 
         raise NotImplementedError
 
@@ -154,6 +181,7 @@ class Value(object):
     methods defined in this base class.
 
     """
+
     _time = 0
 
     def __repr__(self):
@@ -165,12 +193,22 @@ class Value(object):
             self.getUnit(type="prefix"), self.getFactor())
 
     def getDisplayedValue(self):
-        """Returns the displayed value (as a number)."""
+        """Returns the displayed value (as a number).
+
+        :returns: Displayed value
+        :rtype: Float
+
+        """
 
         raise NotImplementedError
 
     def getUnit(self):
-        """Returns the unit (e.g. "g" (gram) or "V" (volt))."""
+        """Returns the unit (e.g. "g" (gram) or "V" (volt)).
+
+        :returns: Unit
+        :rtype: String
+
+        """
 
         raise NotImplementedError
 
@@ -180,17 +218,27 @@ class Value(object):
         For example 10^(-3) for the milli prefix.
         If type == "prefix" it returns the SI-Prefixes.
 
+        :returns: Factor of the displayed value as a number or a SI-Prefix
+        :rtype: Int or String
+
         """
 
         raise NotImplementedError
 
     def getTime(self):
-        """Returns the time the measurement value was taken in seconds since the epoch."""
+        """Returns the time the measurement value was taken in seconds since the epoch.
+    
+        :returns: Time the measurement value was taken
+        :rtype: Float
+
+        """
 
         return self._time
 
 
 class NullValue(Value):
+    """This class represents an "empty" measurement value."""
+
     def getDisplayedValue(self):
         return 0
 
@@ -205,11 +253,7 @@ class NullValue(Value):
 
 
 class TecpelDMM8061(Device):
-    """Multimeter
-
-    TECPEL DMM 8061 or VOLTCRAFT VC 840
-
-    """
+    """This class represents a TECPEL DMM 8061 or VOLTCRAFT VC 840 Multimeter."""
     
     _baudrate = 2400
     _rts = False
@@ -322,7 +366,7 @@ class TecpelDMM8061(Device):
 
             elif type == "unit":
                 if self.ohm:
-                    return "Ω"
+                    return u"Ω"
                 elif self.farad:
                     return "F"
                 elif self.hertz:
@@ -332,28 +376,17 @@ class TecpelDMM8061(Device):
                 elif self.ampere:
                     return "A"
                 else:
-                    return "°C"
+                    return u"°C"
 
         def getFactor(self, type="value"):
             if type == "value":
-                if self.nano:
-                    return pow(10, -9)
-                elif self.micro:
-                    return pow(10, -6)
-                elif self.milli:
-                    return pow(10, -3)
-                elif self.kilo:
-                    return pow(10, 3)
-                elif self.mega:
-                    return pow(10, 6)
-                else:
-                    return pow(10, 0)
+                return si.getFactor(self.getFactor("prefix"))
 
             elif type == "prefix":
                 if self.nano:
                     return "n"
                 elif self.micro:
-                    return "µ"
+                    return u"µ"
                 elif self.milli:
                     return "m"
                 elif self.kilo:
@@ -376,127 +409,127 @@ class TecpelDMM8061(Device):
         else:
             raise Exception
 
-    def getRawValue(self, timeout = 1.5):
+    def getRawValue(self):
         import time
         
         starttime = time.time()
         assert self.isAvailable()
 
-        while not (time.time() - starttime > timeout):
-            read = self._ser.read(26)
-            idList = []
-            for i in read:
-                idList.append(ord(i) >> 4)
+        read = self._ser.read(26)
+        idList = []
+        for i in read:
+            idList.append(ord(i) >> 4)
 
-            try:
-                startIndex = idList.index(1)
+        try:
+            startIndex = idList.index(1)
 
-                # Test whether the read value is correct
-                if idList[startIndex + 1] == 2 and idList[startIndex + 4] == 5 and idList[startIndex + 7] == 8 and\
-                        idList[startIndex + 10] == 11 and idList[startIndex + 12] == 13:
-                    
-                    valueString = read[startIndex:(startIndex+13)]
+            # Test whether the read value is correct
+            if idList[startIndex + 1] == 2 and idList[startIndex + 4] == 5 and idList[startIndex + 7] == 8 and\
+                    idList[startIndex + 10] == 11 and idList[startIndex + 12] == 13:
+                
+                valueString = read[startIndex:(startIndex+13)]
 
-                    result = self.__Value()
+                result = self.__Value()
 
-                    b = ord(valueString[0])
-                    result.rs232  = self._testBit(b, 0)
-                    result.auto  = self._testBit(b, 1)
-                    result.dc  = self._testBit(b, 2)
-                    result.ac = self._testBit(b, 3)
+                b = ord(valueString[0])
+                result.rs232  = self._testBit(b, 0)
+                result.auto  = self._testBit(b, 1)
+                result.dc  = self._testBit(b, 2)
+                result.ac = self._testBit(b, 3)
 
-                    b = ord(valueString[1])
-                    result.d4.a  = self._testBit(b, 0)
-                    result.d4.f  = self._testBit(b, 1)
-                    result.d4.e  = self._testBit(b, 2)
-                    result.d4.ex = self._testBit(b, 3)
+                b = ord(valueString[1])
+                result.d4.a  = self._testBit(b, 0)
+                result.d4.f  = self._testBit(b, 1)
+                result.d4.e  = self._testBit(b, 2)
+                result.d4.ex = self._testBit(b, 3)
 
-                    b = ord(valueString[2])
-                    result.d4.b = self._testBit(b, 0)
-                    result.d4.g = self._testBit(b, 1)
-                    result.d4.c = self._testBit(b, 2)
-                    result.d4.d = self._testBit(b, 3)
+                b = ord(valueString[2])
+                result.d4.b = self._testBit(b, 0)
+                result.d4.g = self._testBit(b, 1)
+                result.d4.c = self._testBit(b, 2)
+                result.d4.d = self._testBit(b, 3)
 
-                    b = ord(valueString[3])
-                    result.d3.a  = self._testBit(b, 0)
-                    result.d3.f  = self._testBit(b, 1)
-                    result.d3.e  = self._testBit(b, 2)
-                    result.d3.ex = self._testBit(b, 3)
+                b = ord(valueString[3])
+                result.d3.a  = self._testBit(b, 0)
+                result.d3.f  = self._testBit(b, 1)
+                result.d3.e  = self._testBit(b, 2)
+                result.d3.ex = self._testBit(b, 3)
 
-                    b = ord(valueString[4])
-                    result.d3.b = self._testBit(b, 0)
-                    result.d3.g = self._testBit(b, 1)
-                    result.d3.c = self._testBit(b, 2)
-                    result.d3.d = self._testBit(b, 3)
+                b = ord(valueString[4])
+                result.d3.b = self._testBit(b, 0)
+                result.d3.g = self._testBit(b, 1)
+                result.d3.c = self._testBit(b, 2)
+                result.d3.d = self._testBit(b, 3)
 
-                    b = ord(valueString[5])
-                    result.d2.a  = self._testBit(b, 0)
-                    result.d2.f  = self._testBit(b, 1)
-                    result.d2.e  = self._testBit(b, 2)
-                    result.d2.ex = self._testBit(b, 3)
+                b = ord(valueString[5])
+                result.d2.a  = self._testBit(b, 0)
+                result.d2.f  = self._testBit(b, 1)
+                result.d2.e  = self._testBit(b, 2)
+                result.d2.ex = self._testBit(b, 3)
 
-                    b = ord(valueString[6])
-                    result.d2.b = self._testBit(b, 0)
-                    result.d2.g = self._testBit(b, 1)
-                    result.d2.c = self._testBit(b, 2)
-                    result.d2.d = self._testBit(b, 3)
+                b = ord(valueString[6])
+                result.d2.b = self._testBit(b, 0)
+                result.d2.g = self._testBit(b, 1)
+                result.d2.c = self._testBit(b, 2)
+                result.d2.d = self._testBit(b, 3)
 
-                    b = ord(valueString[7])
-                    result.d1.a  = self._testBit(b, 0)
-                    result.d1.f  = self._testBit(b, 1)
-                    result.d1.e  = self._testBit(b, 2)
-                    result.d1.ex = self._testBit(b, 3)
+                b = ord(valueString[7])
+                result.d1.a  = self._testBit(b, 0)
+                result.d1.f  = self._testBit(b, 1)
+                result.d1.e  = self._testBit(b, 2)
+                result.d1.ex = self._testBit(b, 3)
 
-                    b = ord(valueString[8])
-                    result.d1.b = self._testBit(b, 0)
-                    result.d1.g = self._testBit(b, 1)
-                    result.d1.c = self._testBit(b, 2)
-                    result.d1.d = self._testBit(b, 3)
+                b = ord(valueString[8])
+                result.d1.b = self._testBit(b, 0)
+                result.d1.g = self._testBit(b, 1)
+                result.d1.c = self._testBit(b, 2)
+                result.d1.d = self._testBit(b, 3)
 
-                    b = ord(valueString[9])
-                    result.diode_test = self._testBit(b, 0)
-                    result.kilo = self._testBit(b, 1)
-                    result.nano = self._testBit(b, 2)
-                    result.micro = self._testBit(b, 3)
+                b = ord(valueString[9])
+                result.diode_test = self._testBit(b, 0)
+                result.kilo = self._testBit(b, 1)
+                result.nano = self._testBit(b, 2)
+                result.micro = self._testBit(b, 3)
 
-                    b = ord(valueString[10])
-                    result.continuity_buzzer = self._testBit(b, 0)
-                    result.mega = self._testBit(b, 1)
-                    result.duty_cycle = self._testBit(b, 2)
-                    result.milli = self._testBit(b, 3)
+                b = ord(valueString[10])
+                result.continuity_buzzer = self._testBit(b, 0)
+                result.mega = self._testBit(b, 1)
+                result.duty_cycle = self._testBit(b, 2)
+                result.milli = self._testBit(b, 3)
 
-                    b = ord(valueString[11])
-                    result.hold = self._testBit(b, 0)
-                    result.delta = self._testBit(b, 1)
-                    result.ohm = self._testBit(b, 2)
-                    result.farad = self._testBit(b, 3)
+                b = ord(valueString[11])
+                result.hold = self._testBit(b, 0)
+                result.delta = self._testBit(b, 1)
+                result.ohm = self._testBit(b, 2)
+                result.farad = self._testBit(b, 3)
 
-                    b = ord(valueString[12])
-                    result.battery_low = self._testBit(b, 0)
-                    result.hertz = self._testBit(b, 1)
-                    result.volt = self._testBit(b, 2)
-                    result.ampere = self._testBit(b, 3)
+                b = ord(valueString[12])
+                result.battery_low = self._testBit(b, 0)
+                result.hertz = self._testBit(b, 1)
+                result.volt = self._testBit(b, 2)
+                result.ampere = self._testBit(b, 3)
 
-                    result._time = time.time()
+                result._time = time.time()
 
-                    return result
-            except (ValueError, IndexError):
-                pass
+                return result
 
-        return None # If timeout is reached
+        except (ValueError, IndexError):
+            pass
+
+        return None
 
 
 class XLS200(MultiboxDevice):
-    """XLS200
+    """This class represents the XLS200 MultiboxDevice.
 
     http://www.xlsmess.de/html/xls_200.html
 
     This device passes one of the three input serial connections, depending
     in which combination DTR and RTS rs232 attributes are set.
 
-    DTR True & RTS False -> 1
-    DTR False & RTS True -> 2
-    DTR False & RTS False -> 3
+    - DTR True & RTS False: Input 1
+    - DTR False & RTS True: Input 2
+    - DTR False & RTS False: Input 3
 
     """
 
@@ -504,25 +537,9 @@ class XLS200(MultiboxDevice):
     _in2 = None
     _in3 = None
 
-    def __init__(self, ser, in1=None, in2=None, in3=None, *args, **kwargs):
-        """Via in{1,2,3} subdevices can be specified.
-
-        This is completely optional and the same result can be achieved
-        by calling openDevice() later.
-
-        """
-
+    def __init__(self, ser):
         super(XLS200, self).__init__(ser)
 
-        if in1:
-            self.openDevice(in1, input = 1, *args, **kwargs)
-
-        if in2:
-            self.openDevice(in2, input = 2, *args, **kwargs)
-
-        if in3:
-            self.openDevice(in3, input = 3, *args, **kwargs)
-
     def __repr__(self):
         string = "{classname}(serial={serial!r}) # ownSer={ownser}".format(
             classname=self.__class__.__name__, serial=self._ser, ownser = self._ownSer)
@@ -549,7 +566,7 @@ class XLS200(MultiboxDevice):
 
         return string + ")"
 
-    def _changeInput(self, input = 1):
+    def _changeInput(self, input):
         # DTR True & RTS False = 1
         # DTR False & RTS True = 2
         # DTR False & RTS False = 3
@@ -574,7 +591,7 @@ class XLS200(MultiboxDevice):
             self._ser.baudrate = self._in3._baudrate
             self._ser.timeout = self._in3._timeout
 
-    def openDevice(self, deviceClass, input=1, *args, **kwargs):
+    def openDevice(self, deviceClass, input, *args, **kwargs):
         assert self.isAvailable()
 
         if input == 1:
@@ -589,55 +606,71 @@ class XLS200(MultiboxDevice):
         else:
             raise Exception("Input has to be in rage of 1 to 3")
 
-    def closeDevice(self, input = 1):
+    def closeDevice(self, input):
         if input == 1:
+            self._in1.close()
             self._in1 = None
 
         elif input == 2:
+            self._in2.close()
             self._in2 = None
 
         elif input == 3:
+            self._in3.close()
             self._in3 = None
 
-    def getRawValue(self, input = 1, timeout = 1.5):
+    def getRawValue(self, input):
         if input == 1:
-            self._changeInput(input = 1)
-            return self._in1.getRawValue(timeout=timeout)
+            self._changeInput(1)
+            return self._in1.getRawValue()
 
         elif input == 2:
-            self._changeInput(input = 2)
-            return self._in2.getRawValue(timeout=timeout)
+            self._changeInput(2)
+            return self._in2.getRawValue()
 
         elif input == 3:
-            self._changeInput(input = 3)
-            return self._in3.getRawValue(timeout=timeout)
+            self._changeInput(3)
+            return self._in3.getRawValue()
 
-    def getDevice(self, input = 1):
+    def getDevice(self, input):
         if input == 1:
-            self._changeInput(input = 1)
+            self._changeInput(1)
             return self._in1
 
         elif input == 2:
-            self._changeInput(input = 2)
+            self._changeInput(2)
             return self._in2
 
         elif input == 3:
-            self._changeInput(input = 3)
+            self._changeInput(3)
             return self._in3
 
 
 class Balance(Device):
+    """This class represents Balance Devices."""
     _typeOfValue = "all"
 
-    def __init__(self, ser, typeOfValue="all", *args, **kwargs):
-        super(Balance, self).__init__(ser)
+    def setTypeOfValue(self, typeOfValue):
+        """Balance Devices have two different kind of values: Stable and unstable Values.
+
+        This method sets, which kind of values should be returned by the getRawValue method.
+        For more information please see the Device's manual.
+
+        :param typeOfValue: Type of values, which should be returned by the getRawValue method - "stable" or "all"
+        :type typeOfValue: String
+
+        """
 
         if typeOfValue in ["stable", "all"]:
             self._typeOfValue = typeOfValue
         else:
             raise Exception("Invalid typeOfValue")
 
+
+
 class KernPCB(Balance):
+    """This class represents the KernPCB Balance."""
+
     import re
     _regex = re.compile(r"\s[\s-][\d\s\.]{10}\s.{3}\r\n")
 
@@ -668,7 +701,7 @@ class KernPCB(Balance):
             elif type == "prefix":
                 return ""
 
-    def getRawValue(self, timeout = 1.5):
+    def getRawValue(self):
         import time
 
         starttime = time.time()
@@ -678,40 +711,38 @@ class KernPCB(Balance):
         result = self.__Value()
 
         if self._typeOfValue == "stable":
-            while True and not (time.time() - starttime > timeout):
-                self._ser.write("s")
-                s = self._ser.read(size = 35)
-                # One value has 18 bytes, to make sure to get a complete one
-                # (and not the second 1/2 of one, and the first 1/2 of another)
-                # we read 35 bytes and match for one complete value
+            self._ser.write("s")
+            s = self._ser.read(size = 35)
+            # One value has 18 bytes, to make sure to get a complete one
+            # (and not the second 1/2 of one, and the first 1/2 of another)
+            # we read 35 bytes and match for one complete value
 
-                try:
-                    result.string = self._regex.search(s).group()
-                    result._time = time.time()
-                    return result
+            try:
+                result.string = self._regex.search(s).group()
+                result._time = time.time()
+                return result
 
-                except AttributeError:
-                    pass
+            except AttributeError:
+                pass
 
-            return None # return None, if timeout is reached
+            return None
 
         elif self._typeOfValue == "all":
-            while True and not (time.time() - starttime > timeout):
-                self._ser.write("w")
-                s = self._ser.read(size = 35)
-                # One value has 18 bytes, to make sure to get a complete one
-                # (and not the second 1/2 of one, and the first 1/2 of another)
-                # we read 35 bytes and match for one complete value
+            self._ser.write("w")
+            s = self._ser.read(size = 35)
+            # One value has 18 bytes, to make sure to get a complete one
+            # (and not the second 1/2 of one, and the first 1/2 of another)
+            # we read 35 bytes and match for one complete value
 
-                try:
-                    result.string = self._regex.search(s).group()
-                    result._time = time.time()
-                    return result
+            try:
+                result.string = self._regex.search(s).group()
+                result._time = time.time()
+                return result
 
-                except AttributeError:
-                    pass
+            except AttributeError:
+                pass
 
-            return None # return None, if timeout is reached
+            return None
 
     def setTara(self):
         """Set taring.
@@ -726,6 +757,8 @@ class KernPCB(Balance):
 
 
 class BS600(Balance):
+    """This class represents the BS600 Balance."""
+
     import re
     _stable = re.compile(r"[WCP][TC]ST[+-][\d\s\.]{7}.{4}")
     _all = re.compile(r"[WCP][TC][SU][TS][+-][\d\s\.]{7}.{4}")
@@ -756,7 +789,7 @@ class BS600(Balance):
             elif type == "prefix":
                 return ""
     
-    def getRawValue(self, timeout = 1.5):
+    def getRawValue(self):
         import time
 
         starttime = time.time()
@@ -765,24 +798,23 @@ class BS600(Balance):
 
         result = self.__Value()
 
-        while True and not (time.time() - starttime > timeout):
-            val = self._ser.read(size = 35)
-            # One value has 18 bytes, to make sure to get a complete one
-            # (and not the second 1/2 of one, and the first 1/2 of another)
-            # we read 35 bytes and match for one complete value
+        val = self._ser.read(size = 35)
+        # One value has 18 bytes, to make sure to get a complete one
+        # (and not the second 1/2 of one, and the first 1/2 of another)
+        # we read 35 bytes and match for one complete value
 
-            try:
-                if self._typeOfValue == "stable":
-                    result.string = self._stable.search(val).group()
-                    result._time = time.time()
-                    return result
+        try:
+            if self._typeOfValue == "stable":
+                result.string = self._stable.search(val).group()
+                result._time = time.time()
+                return result
 
-                elif self._typeOfValue == "all":
-                    result.string = self._all.search(val).group()
-                    result._time = time.time()
-                    return result
+            elif self._typeOfValue == "all":
+                result.string = self._all.search(val).group()
+                result._time = time.time()
+                return result
 
-            except AttributeError:
-                pass
+        except AttributeError:
+            pass
 
-        return None # return None, if timeout is reached
\ No newline at end of file
+        return None
