@@ -18,13 +18,22 @@
 
 import sys
 import pkgutil
-import StringIO
 import qr
 from PyQt4 import QtCore, QtGui, uic
-from devices.devicemanager import DeviceManager, DeviceConfig
+from devices.devicemanager import DeviceManager
+import devices.si as si
 
 class NewDeviceDialog(QtGui.QDialog):
+    """Dialog to add new devices."""
+
     def __init__(self, dm, parent=None):
+        """
+
+        :param dm: DeviceManager
+        :type dm: DeviceManager
+
+        """
+
         QtGui.QDialog.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/newDeviceDialog.ui")
         qfile.open(QtCore.QIODevice.ReadOnly)
@@ -36,7 +45,18 @@ class NewDeviceDialog(QtGui.QDialog):
 
 
 class DoReallyDialog(QtGui.QDialog):
+    """Dialog to ask whether a user wants really to do something."""
+
     def __init__(self, title, text, parent=None):
+        """
+
+        :param title: Dialog title
+        :type title: String
+        :param text: Dialog text
+        :type text: String
+
+        """
+
         QtGui.QDialog.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/doReallyDialog.ui")
         qfile.open(QtCore.QIODevice.ReadOnly)
@@ -47,6 +67,8 @@ class DoReallyDialog(QtGui.QDialog):
 
 
 class Xls200Dialog(QtGui.QDialog):
+    """Dialog to chose XLS200 subdevices."""
+
     def __init__(self, parent=None):
         QtGui.QDialog.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/xls200Dialog.ui")
@@ -57,6 +79,8 @@ class Xls200Dialog(QtGui.QDialog):
 
 
 class SettingsDialog(QtGui.QDialog):
+    """The general settings dialog."""
+
     def __init__(self, parent=None):
         QtGui.QDialog.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/settingsDialog.ui")
@@ -69,61 +93,51 @@ class SettingsDialog(QtGui.QDialog):
         self.settings = QtCore.QSettings()
 
         self.path.setText(self.settings.value("office/path", "").toString())
-        self.loggingInterval.setValue(self.settings.value("logging/interval", 1).toInt()[0])
         self.languageComboBox.setCurrentIndex(self.settings.value("i18n", -1).toInt()[0])
 
     def openFile(self):
+        """Open a QFileDialog and save the path."""
+
         import os
         popup = QtGui.QFileDialog()
         self.path.setText(popup.getOpenFileName(self, self.tr("Search Office"), os.path.expanduser("~"), ""))
 
     def save(self):
+        """Save the settings to QSettings."""
+
         self.settings.setValue("office/path", self.path.text())
-        self.settings.setValue("logging/interval", self.loggingInterval.value())
         self.settings.setValue("i18n", self.languageComboBox.currentIndex())
 
 
-class DevicemanagerDialog(QtGui.QDialog):
-    ids = []
-
-    def __init__(self, dm, parent=None):
-        QtGui.QDialog.__init__(self, parent)
-        qfile = QtCore.QFile(":/ui/devicemanagerDialog.ui")
-        qfile.open(QtCore.QIODevice.ReadOnly)
-        self.ui = uic.loadUi(qfile, self)
-        self.dm = dm
-
-        self.deleteButton.clicked.connect(self.deleteItem)
-        self.deleteButton.setIcon(QtGui.QIcon(":/images/close.png"))
-
-        self.refreshList()
-
-    def refreshList(self):
-        self.ids = self.dm.getAllDeviceIDs()
-
-        self.listWidget.clear()
-
-        for id in self.ids:
-            item = QtGui.QListWidgetItem(str(self.dm.getDevice(id)))
-            self.listWidget.addItem(item)
-
-    def deleteItem(self):
-        row = self.listWidget.currentRow()
-
-        self.dm.closeDevice(self.ids[row])
-        self.dm.closeEmptyMultiboxDevices()
-
-        self.refreshList()
-
-
 class DisplayWidget(QtGui.QWidget):
+    """Widget containing a lcd display and buttons to modify or delete a device."""
     calibrationType = 1 # 0: two values calibration, 1: slope and intercept calibration
     twoValueCalibration = (0.0, 0.0), (1.0, 1.0)
     slopeInterceptCalibration = 1.0, 0.0
-    calibration = 1.0, 0.0 # this is either the same as slopeInterceptCalibration or twoValueCalibration
-    unit = ""
+
+    is1PrefixIndex = 2 # QComboBox Index
+    should1PrefixIndex = 2 # QComboBox Index
+    is2PrefixIndex = 2 # QComboBox Index
+    should2PrefixIndex = 2 # QComboBox Index
+
+    is1 = 0
+    should1 = 0
+    is2 = 1
+    should2 = 1
+
+    calibration = slopeInterceptCalibration # either slopeInterceptCalibration or twoValueCalibration
+    unit = QtCore.QString()
 
     def __init__(self, deviceID, dm, parent=None):
+        """
+
+        :param deviceID: DeviceID
+        :type deviceID: DeviceID
+        :param dm: DeviceManager
+        :type dm: DeviceManager
+
+        """
+
         QtGui.QWidget.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/displayWidget.ui")
         qfile.open(QtCore.QIODevice.ReadOnly)
@@ -140,14 +154,26 @@ class DisplayWidget(QtGui.QWidget):
         self.deleteButton.setIcon(QtGui.QIcon(":/images/close.png"))
 
     def deviceSettings(self):
+        """Open a DeviceSettingsDialog."""
+
         popup = DeviceSettingsDialog(self.deviceID, self.dm)
 
         popup.slope.setValue(self.slopeInterceptCalibration[0])
         popup.intercept.setValue(self.slopeInterceptCalibration[1])
-        popup.is1.setValue(self.twoValueCalibration[0][0])
-        popup.should1.setValue(self.twoValueCalibration[0][1])
-        popup.is2.setValue(self.twoValueCalibration[1][0])
-        popup.should2.setValue(self.twoValueCalibration[1][1])
+        popup.is1.setValue(self.is1)
+        popup.should1.setValue(self.should1)
+        popup.is2.setValue(self.is2)
+        popup.should2.setValue(self.should2)
+
+        # Add prefixes to combobox
+        for i in [popup.is1Prefix, popup.should1Prefix, popup.is2Prefix, popup.should2Prefix]:
+            i.clear()
+            i.addItems(si.getSiNames())
+
+        popup.is1Prefix.setCurrentIndex(self.is1PrefixIndex)
+        popup.should1Prefix.setCurrentIndex(self.should1PrefixIndex)
+        popup.is2Prefix.setCurrentIndex(self.is2PrefixIndex)
+        popup.should2Prefix.setCurrentIndex(self.should2PrefixIndex)
 
         if self.calibrationType == 1:
             popup.slopeInterceptButton.setChecked(True)
@@ -158,9 +184,23 @@ class DisplayWidget(QtGui.QWidget):
 
         popup.exec_()
 
-        self.twoValueCalibration = (popup.is1.value(), popup.should1.value()), (popup.is2.value(), popup.should2.value())
+        self.twoValueCalibration = ((popup.is1.value() * si.getFactor(str(popup.is1Prefix.currentText())),
+                popup.should1.value()  * si.getFactor(str(popup.should1Prefix.currentText()))),
+            (popup.is2.value() * si.getFactor(str(popup.is2Prefix.currentText())), 
+                popup.should2.value() * si.getFactor(str(popup.should2Prefix.currentText()))))
+
         self.slopeInterceptCalibration = popup.slope.value(), popup.intercept.value()
 
+        self.is1 = popup.is1.value()
+        self.should1 = popup.should1.value()
+        self.is2 = popup.is2.value()
+        self.should2 = popup.should2.value()
+
+        self.is1PrefixIndex = popup.is1Prefix.currentIndex()
+        self.should1PrefixIndex = popup.should1Prefix.currentIndex()
+        self.is2PrefixIndex = popup.is2Prefix.currentIndex()
+        self.should2PrefixIndex = popup.should2Prefix.currentIndex()
+
         if popup.slopeInterceptButton.isChecked():
             self.calibrationType = 1
             self.calibration = self.slopeInterceptCalibration
@@ -168,12 +208,16 @@ class DisplayWidget(QtGui.QWidget):
             self.calibrationType = 0
             self.calibration = self.twoValueCalibration
 
-        self.unit = str(popup.unit.text())
+        self.unit = popup.unit.text()
 
     def delete(self):
+        """Delete Widget."""
+
         self.deleteLater()
 
     def close(self):
+        """Close the device."""
+
         if self.dm.getStatus() == False:
             self.dm.closeDevice(self.deviceID)
         else:
@@ -183,7 +227,17 @@ class DisplayWidget(QtGui.QWidget):
 
 
 class DeviceSettingsDialog(QtGui.QDialog):
+    """Settings dialog for device specific settings."""
+
     def __init__(self, deviceID, dm, parent=None):
+        """
+
+        :param deviceID: DeviceID
+        :type deviceID: DeviceID
+        :param dm: DeviceManager
+        :type dm: DeviceManager
+
+        """
         QtGui.QDialog.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/deviceSettingsDialog.ui")
         qfile.open(QtCore.QIODevice.ReadOnly)
@@ -196,15 +250,31 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.get2.clicked.connect(self.setCurrentValue2)
 
     def setCurrentValue1(self):
+        """Get current displayedValue from Device."""
+
         rv = self.dm.getLastRawValue(self.deviceID)
         self.is1.setValue(rv.getDisplayedValue())
 
+        name = si.getName(rv.getFactor())
+        index = self.is1Prefix.findText(name)
+        self.is1Prefix.setCurrentIndex(index)
+        self.should1Prefix.setCurrentIndex(index)
+
     def setCurrentValue2(self):
+        """Get current displayedValue from Device."""
+
         rv = self.dm.getLastRawValue(self.deviceID)
         self.is2.setValue(rv.getDisplayedValue())
 
+        name = si.getName(rv.getFactor())
+        index = self.is1Prefix.findText(name)
+        self.is2Prefix.setCurrentIndex(index)
+        self.should2Prefix.setCurrentIndex(index)
+
 
 class MainWindow(QtGui.QMainWindow):
+    """Main window."""
+
     dm = None
     displayWidgets = {}
 
@@ -214,7 +284,6 @@ class MainWindow(QtGui.QMainWindow):
     tmpfile = None
     starttime = 0
     lasttime = 0
-    loggingInterval = 3
     pathToLogFile = None
 
     officePath = None
@@ -225,6 +294,8 @@ class MainWindow(QtGui.QMainWindow):
         qfile.open(QtCore.QIODevice.ReadOnly)
         self.ui = uic.loadUi(qfile, self)
 
+        self.settings = QtCore.QSettings()
+
         self.measurementButton.clicked.connect(self.startStopMeasurement)
         self.loggingButton.clicked.connect(self.startStopLogging)
         self.saveButton.clicked.connect(self.saveLog)
@@ -232,7 +303,6 @@ class MainWindow(QtGui.QMainWindow):
         self.openButton.clicked.connect(self.openLog)
 
         self.actionSettings.triggered.connect(self.settingsDialog)
-        self.actionDevicemanager.triggered.connect(self.devicemanagerDialog)
 
         self.dm = DeviceManager()
         self.timer = QtCore.QTimer()
@@ -241,25 +311,24 @@ class MainWindow(QtGui.QMainWindow):
 
         self.settings = QtCore.QSettings()
         self.officePath = str(self.settings.value("office/path", "").toString())
-        self.loggingInterval = int(self.settings.value("logging/interval", 1).toInt()[0])
+
+        self.loggingInterval.setValue(self.settings.value("logging/interval", 1).toInt()[0])
+        self.loggingInterval.valueChanged.connect(self.saveLoggingInterval)
+
+    def saveLoggingInterval(self):
+        self.settings.setValue("logging/interval", self.loggingInterval.value())
 
     def settingsDialog(self):
+        """Open a SettingsDialog."""
+
         popup = SettingsDialog()
         popup.exec_()
 
         self.officePath = str(self.settings.value("office/path", "").toString())
-        self.loggingInterval = int(self.settings.value("logging/interval", 1).toInt()[0])
-
-    def devicemanagerDialog(self):
-        if self.dm.getStatus() == False:
-            popup = DevicemanagerDialog(self.dm)
-            popup.exec_()
-        else:
-            popup = DoReallyDialog(self.tr("Warning"),
-                self.tr("You have to stop the measurement to open the devicemanager."))
-            popup.exec_()
 
     def openLog(self):
+        """Open last log with office."""
+        
         import subprocess
         import os
 
@@ -273,6 +342,8 @@ class MainWindow(QtGui.QMainWindow):
             popup.exec_()
 
     def addDevice(self):
+        """Open a NewDeviceDialog."""
+        
         popup = NewDeviceDialog(self.dm)
         popup.exec_()
 
@@ -289,36 +360,38 @@ class MainWindow(QtGui.QMainWindow):
                 xls200Popup.exec_()
 
                 if xls200Popup.result():
-                    xls200ID = self.dm.openWithConfig(DeviceConfig((port, {}), device))
+                    xls200ID = self.dm.openDevice(device, port)
                     sub1 = str(xls200Popup.subdevice1ComboBox.currentText())
                     sub2 = str(xls200Popup.subdevice2ComboBox.currentText())
                     sub3 = str(xls200Popup.subdevice3ComboBox.currentText())
                     
                     if sub1 != "":
-                            deviceID = self.dm.openWithConfig(DeviceConfig((xls200ID, {}, 1), sub1))
+                            deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
                             sub1Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub1Widget)
                             self.displayWidgets[deviceID] = sub1Widget
 
                     if sub2 != "":
-                            deviceID = self.dm.openWithConfig(DeviceConfig((xls200ID, {}, 2), sub2))
+                            deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
                             sub2Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub2Widget)
                             self.displayWidgets[deviceID] = sub2Widget
 
                     if sub3 != "":
-                            deviceID = self.dm.openWithConfig(DeviceConfig((xls200ID, {}, 3), sub3))
+                            deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
                             sub3Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub3Widget)
                             self.displayWidgets[deviceID] = sub3Widget
 
             else:
-                deviceID = self.dm.openWithConfig(DeviceConfig((port, {}), device))
+                deviceID = self.dm.openDevice(device, port)
                 deviceWidget = DisplayWidget(deviceID, self.dm)
                 self.verticalLayout.addWidget(deviceWidget)
                 self.displayWidgets[deviceID] = deviceWidget
 
     def startStopMeasurement(self):
+        """Start/Stop measurement."""
+        
         if self.dm.getStatus() == False:
             self.dm.start()
             self.running = True
@@ -330,6 +403,8 @@ class MainWindow(QtGui.QMainWindow):
             self.measurementButton.setText(self.tr("Start"))
 
     def startStopLogging(self):
+        """Start/Stop logging."""
+
         import time
         import tempfile
 
@@ -353,6 +428,8 @@ class MainWindow(QtGui.QMainWindow):
             self.loggingButton.setText(self.tr("Start logging"))
 
     def saveLog(self):
+        """Save last log to file."""
+
         import os
 
         if self.tmpfile:
@@ -375,6 +452,8 @@ class MainWindow(QtGui.QMainWindow):
                 return
 
     def update(self):
+        """Update the displayWidgets and, when logging enabled log."""
+        
         import time
         # python3 incompatibility: .iteritems()
         # Delete unnecessary widgets
@@ -402,11 +481,10 @@ class MainWindow(QtGui.QMainWindow):
 
                 rv = self.dm.getCalibratedLastRawValue(widget.deviceID, widget.calibration, widget.unit)
                 widget.lcdNumber.display(rv.getDisplayedValue())
-                widget.label.setText(str(rv.getFactor("prefix") + rv.getUnit()).decode('utf-8'))
-                # python3 incompatibility: in python3 .decode() is not needed anymore
+                widget.label.setText(rv.getFactor("prefix") + rv.getUnit())
             
             # log    
-            if self.log and ((time.time() - self.lasttime) > self.loggingInterval):
+            if self.log and ((time.time() - self.lasttime) > self.loggingInterval.value()):
                 for deviceID, widget in self.displayWidgets.iteritems():
                     unit = None
                     if widget.unit != "":
